# frozen_string_literal: true

require 'streamio-ffmpeg'
require 'date'

module Zashoku
  module AnimeIO
    class AnimeFile < Zashoku::Item
      include Zashoku::Util

      EMPTY_ATTR = {
        't' => 'empty',
        'd' => 0,
        'D' => '0',
        's' => 0,
        'S' => '0',
        'r' => 0x0,
        'a' => 'n.a.',
        'v' => '',
        'l' => '',
        'f' => '',
        'b' => '',
      }.freeze

      attr_accessor :last_modified, :accessible, :path, :watched_on, :attr

      def initialize(path, attr = {})
        @watched    = attr['watched'] || false
        @watched_on = attr['watched_on'] || 0
        @path       = path

        begin
          @last_modified = File::Stat.new(path).mtime
        rescue Errno::ENOENT
          @last_modified = '0'
        end

        @attr = (@path != 'empty') ? probe_file : EMPTY_ATTR
      end

      def attr
        @attr.merge({
          'w' => watched? ? Zashoku.conf.get(%w[animeio icons watched]) : Zashoku.conf.get(%w[animeio icons unwatched])
        })
      end

      def probe_file
        mov = FFMPEG::Movie.new(@path)
        {
          't' => pretty_string,
          'd' => mov.duration,
          'D' => Zashoku::Formatter.duration(mov.duration),
          's' => mov.size,
          'S' => Zashoku::Formatter.size(mov.size),
          'r' => mov.resolution,
          'a' => mov.audio_codec,
          'v' => mov.video_codec,
          'l' => mov.colorspace,
          'f' => mov.frame_rate,
          'b' => String(mov.bitrate) + 'kb/s',
        }
      end

      def title
        @attr['t']
      end

      def watched?
        @watched
      end

      def watched!
        @watched_on = DateTime.now.strftime('%Q')
        @attr['w']  = 'x'
        @watched    = true
      end

      def unwatch
        @watched   = false
      end

      def pretty_string
        @string || @string = Zashoku::Formatter.media_file(path)
      end

      def to_s
        path
      end
    end
  end
end
