# frozen_string_literal: true

require 'date'

module Zashoku
  module AnimeIO
    class AnimeSeries < Zashoku::Item
      include Zashoku::Util

      attr_accessor :attr

      def initialize(path, children = [])
        @path = path

        @attr = {
          't' => String(@path),
          'c' => 0,
          'd' => 0,
          's' => 0,
          'p' => 0,
          'w' => 0,
          'W' => 0
        }

        @children = children
        @children.each { |c| self + c }
      end

      def +(other)
        @attr['d'] += other.attr['d']
        @attr['D']  = Zashoku::Formatter.duration(@attr['d'])
        @attr['s'] += other.attr['s']
        @attr['S']  = Zashoku::Formatter.size(@attr['s'])
        @attr['c'] += 1
        @attr['w'] += 1 if other.watched?
        @attr['W'] += other.attr['d'] if other.watched?
      end

      def <<(af)
        @children << af
        self + af
        average
      end

      def average
        @attr['p'] = @attr['w'] * 100.0 / @attr['c']
        @attr['q'] = @attr['d'].positive? ? @attr['W'] * 100.0 / @attr['d'] : 0
        @attr['Q'] = Zashoku::Formatter.progress @attr['q']
        @attr['P'] = Zashoku::Formatter.progress @attr['p']
      end

      def title
        @path
      end

      def attributes
        @attr
      end
    end
  end
end
