# frozen_string_literal: true

module Zashoku
  module Mpv
    class MpvController
      include Observable

      DefaultFilename = '.'
      DefaultTimeHash = { pos: 0, end: 0, pct: 0, lu: Time.now}

      attr_accessor :mpv, :filename, :time

      def initialize
        defaults
        connect
      end

      def defaults
        @filename = DefaultFilename
        @time = DefaultTimeHash.dup
      end

      def refresh_filename
        changed
        notify_observers(
          sender: 'mpv',
          event: 'refresh_widget',
          statusline: true,
          widget: 'filename'
        )
      end

      def refresh_time
        changed
        notify_observers(
          sender: 'mpv',
          event: 'refresh_widget',
          statusline: true,
          widget: 'time'
        )
      end

      def connect
        return unless @mpv&.client&.get_property("idle-active") == nil

        @mpv = MPV::Session.new(
          user_args: Zashoku.conf.get(['mpv_args']).split
        )
        @mpv.callbacks << method(:event_handler)
        @mpv.client.command('observe_property_string', 2, 'time-pos')
        @mpv.client.command('observe_property_string', 1, 'path')
      end

      def event_handler(e)
        # p e
        case e['event']
        when 'property-change'
          case e['name']
          when 'path'
            @filename = e['data'] || DefaultFilename
            refresh_filename
          when 'time-pos'
            return if Time.now < @time[:lu] + 1
            @time[:pos] = e['data']
            @time[:end] = @mpv.client.get_property('duration')
            @time[:pct] = @mpv.client.get_property('percent-pos')
            @time[:lu] = Time.now
            refresh_time
          end
        when "end-file"
          defaults
          refresh_time
          refresh_filename
        end
      end

      def watch(file)
        connect
        @mpv.client.command('loadfile', file)
        Zashoku.command(mod: 'animeio', meth: 'watched!', args: [file])
      end
    end
  end
end

