# frozen_string_literal: true
require 'securerandom'
require 'fileutils'

module Zashoku
  module MPVCSide
    class MPVCSideView < Zashoku::View
      TMP_FILE = '/tmp/cside-stream-' + SecureRandom.uuid

      def initialize
        @tmp_file = TMP_FILE + '-' + SecureRandom.uuid
        super
      end

      def cleanup
        FileUtils.rm(@tmp_file)
      rescue Errno::ENOENT
        Zashoku.logger.warn("mpvcside: file #{@tmp_file} does not exist")
      end

      def watch(file)
        Zashoku::Util.alert(TMP_FILE)
        Thread.new do
          Zashoku.command(
            mod: 'remote::mpvcside',
            meth: 'watch',
            args: [file],
            to_file: @tmp_file
          )
        end
        sleep 0.1
        while File.stat(@tmp_file).size < 1
          sleep 0.1
        end
        play_tmp
      end

      def play_tmp
        connect
        @mpv.client.command('loadfile', @tmp_file)
      end

      def connect
        return unless @mpv&.client&.get_property('idle-active') == nil

        @mpv = MPV::Session.new(
          user_args: []
        )
      end

      def refresh_attributes; end
      def refresh_items; {} end
      def get_format; {'title' => ''} end
    end
  end
end
